<?php

namespace Drupal\tranc;

use Drupal\Core\Template\TwigNodeTrans;
use Twig\Environment;
use Twig\Node\Expression\ArrayExpression;
use Twig\Node\Expression\ConstantExpression;
use Twig\Node\Expression\FilterExpression;
use Twig\Node\Node;
use Twig\NodeVisitor\AbstractNodeVisitor;

/**
 * Changes the t and trans filters to tc in the default theme.
 */
class TrancNodeVisitor extends AbstractNodeVisitor {

  /**
   * The default theme path.
   *
   * @var string
   */
  protected $path;

  /**
   * TrancNodeVisitor constructor.
   *
   * @param string $path
   *   The path to the default theme.
   */
  public function __construct($path) {
    $this->path = $path;
  }

  /**
   * {@inheritdoc}
   */
  protected function doEnterNode(Node $node, Environment $env) {
    return $node;
  }

  /**
   * {@inheritdoc}
   */
  protected function doLeaveNode(Node $node, Environment $env) {
    if (strpos(realpath((string) $node->getTemplateName()), $this->path) === 0) {
      if ($node instanceof FilterExpression) {
        $name = $node->getNode('filter')->getAttribute('value');
        if ($name === 't' || $name === 'trans') {
          // Use our own translation filter.
          $node->getNode('filter')->setAttribute('value', 'tc');
        }
      }
      elseif ($node instanceof TwigNodeTrans) {
        $lineno = $node->getTemplateLine();
        if ($node->hasNode('options')) {
          $options = $node->getNode('options');
        }
        else {
          $options = new ArrayExpression([], $lineno);
          $node->setNode('options', $options);
        }
        $langcodeExpression = new ConstantExpression('langcode', $lineno);
        if (!$options->hasElement($langcodeExpression)) {
          $contentExpression = new ConstantExpression(TrancTranslationManager::CONTENT_LANGCODE, $lineno);
          $options->addElement($contentExpression, $langcodeExpression);
        }
      }
    }
    return $node;
  }

  /**
   * {@inheritdoc}
   */
  public function getPriority() {
    // Copied from Drupal\Core\Template\TwigNodeVisitor.
    return 256;
  }

}
